package de.wenzlaff.bibelleseplan;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.wenzlaff.twbibel.Bibel;
import de.wenzlaff.twbibel.Bibelbuecher;

/**
 * Bibelleseplan Generator um die Bibel in einem Jahr zu lesen.
 * 
 * Default: 3 Kapitel pro Tag und am Wochenende 4 Kapitel pro Tag
 * 
 * @author Thomas Wenzlaff
 *
 */
public class Generator {

	private static final Logger LOG = LogManager.getLogger(Generator.class);

	/** Kapitel pro Wochentag die maximal gelesen werden. */
	private static final int DEFAULT_KAPITEL_PRO_TAG = 3;
	/** Kapitel die am Samstag und Sonntag maximal gelesen werden. */
	private static final int DEFAULT_KAPITEL_PRO_WOCHENEND = 4;

	/** Maximale Tage die benötigt werden. Mind. ein Kapitel pro Tag. */
	private static final int MAXIMALE_TAGE = Bibel.maxKapitel();

	private static int kapitel_counter = 0;
	private static int tag_counter = 0;

	/**
	 * Liefert den Bibelleseplan für ein Buch oder die ganze Bibel (null).
	 * 
	 * @param bibelbuch  oder null für die ganze Bibel
	 * @param startDatum das Startdatum
	 * @return der Plan zum lesen
	 */
	public static List<Plan> getBibelleseplan(Bibelbuecher bibelbuch, LocalDate startDatum) {

		List<Plan> bibellesePlan = new ArrayList<>();

		LocalDate startDate = LocalDate.of(startDatum.getYear(), startDatum.getMonth(), startDatum.getDayOfMonth());

		LocalDate endDate = startDate.plusDays(MAXIMALE_TAGE);

		List<LocalDate> erg = getDatesBetween(startDate, endDate);

		if (bibelbuch == null) {

			Bibel.getAlleKapitel().stream().forEach(buch -> {

				LOG.debug(erg.get(tag_counter) + " , " + buch);
				bibellesePlan.add(new Plan(erg.get(tag_counter), buch));

				nextTag(erg);
			});
			LOG.info("Liefere Plan für die ganze Bibel.");
		} else {
			Bibel.getAlleKapitel(bibelbuch).stream().forEach(buch -> {

				LOG.debug(erg.get(tag_counter) + " , " + buch);
				bibellesePlan.add(new Plan(erg.get(tag_counter), buch));

				nextTag(erg);
			});
			LOG.info("Liefere Plan für das Bibelbuch: " + bibelbuch.name());
		}

		return bibellesePlan;
	}

	private static void nextTag(List<LocalDate> erg) {

		kapitel_counter++;

		if (isWochenende(erg)) {
			if (kapitel_counter == DEFAULT_KAPITEL_PRO_WOCHENEND) {
				tag_counter++;
				kapitel_counter = 0;
			}
		} else {
			if (kapitel_counter == DEFAULT_KAPITEL_PRO_TAG) {
				tag_counter++;
				kapitel_counter = 0;
			}
		}
	}

	private static boolean isWochenende(List<LocalDate> erg) {
		return erg.get(tag_counter).getDayOfWeek().equals(DayOfWeek.SATURDAY) || erg.get(tag_counter).getDayOfWeek().equals(DayOfWeek.SUNDAY);
	}

	/**
	 * Liefert alle Tage zwischen start und ende Datum.
	 * 
	 * Berücksichtigt Schaltjahr!
	 * 
	 * @param startDate
	 * @param endDate
	 * @return alle Datum von bis.
	 */
	private static List<LocalDate> getDatesBetween(LocalDate startDate, LocalDate endDate) {

		long daysBetween = ChronoUnit.DAYS.between(startDate, endDate);

		return IntStream.iterate(0, i -> i + 1).limit(daysBetween).mapToObj(i -> startDate.plusDays(i)).collect(Collectors.toList());
	}

	public static String getIcsDateinamen(Parameter parameter) {
		String name;
		if (parameter.getBibelbuch() == null) {
			name = "alle";
		} else {
			name = parameter.getBibelbuch().name();
		}

		String dateinamen = "bibelleseplan_" + name + "_" + LocalDateTime.now().toString().replace(':', '_').replace('.', '_') + ".ics";
		return dateinamen;
	}

}
